local uabStatus_MissingBuffs = uabUnitStatus:NewModule("Missing Buffs")

uabStatus_MissingBuffs.userdefined = {}
uabStatus_MissingBuffs.mixins = {}
uabStatus_MissingBuffs.mixins.aura = AceLibrary("SpecialEvents-Aura-2.0")
uabStatus_MissingBuffs.defaultDB = {
    disabled = nil,
    intellect = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = nil,
            shaman = true,
            warlock = true,
            warrior = nil,
        },
	},
    fort = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    shadowprot = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    spirit = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = nil,
            shaman = true,
            warlock = true,
            warrior = nil,
        },
	},
    mark = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    
    userdefined = {
    },
}

uabStatus_MissingBuffs.constants = {
    strings = {
        bintellect = "Missing: Arcane Intellect",
        bfort = "Missing: Fortitude",
        bshadowprot = "Missing: Shadow Protection",
        bspirit = "Missing: Divine Spirit",
        bmark = "Missing: Mark of the Wild",
    
        intellect1 = "Arcane Intellect",
        intellect2 = "Arcane Brilliance",
        fort1 = "Power Word: Fortitude",
        fort2 = "Prayer of Fortitude",
        shadowprot = "Shadow Protection",
        spirit1 = "Divine Spirit",
        spirit2 = "Prayer of Spirit",
        mark1 = "Mark of the Wild",
        mark2 = "Gift of the Wild",
    }
}

function uabStatus_MissingBuffs:OnInitialize()
    self.super.OnInitialize(self)

	self:RegisterUnitStatus("Missing Buffs", uabStatus_MissingBuffs.constants.strings.bintellect)
	self:RegisterUnitStatus("Missing Buffs", uabStatus_MissingBuffs.constants.strings.bfort)
	self:RegisterUnitStatus("Missing Buffs", uabStatus_MissingBuffs.constants.strings.bshadowprot)
	self:RegisterUnitStatus("Missing Buffs", uabStatus_MissingBuffs.constants.strings.bspirit)
	self:RegisterUnitStatus("Missing Buffs", uabStatus_MissingBuffs.constants.strings.bmark)

    self:RegisterEvent("SpecialEvents_UnitBuffGained")
    self:RegisterEvent("SpecialEvents_UnitBuffLost")
    
    -- dynamically added buffs
    for name, settings in pairs(self.db.profile.userdefined) do
        self:RegisterUnitStatus("Missing Buffs", name)
    end
end

function uabStatus_MissingBuffs:SpecialEvents_UnitBuffGained(unit, buffName, index, applications, texture, rank)
    local class = self:GetUnitClass(unit)
    local hasbuff
    
	if buffName == uabStatus_MissingBuffs.constants.strings.intellect1 or buffName == uabStatus_MissingBuffs.constants.strings.intellect2 then
        if not class or self.db.profile.intellect.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bintellect
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.fort1 or buffName == uabStatus_MissingBuffs.constants.strings.fort2 then
        if not class or self.db.profile.fort.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bfort
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.shadowprot then
        if not class or self.db.profile.shadowprot.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bshadowprot
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.spirit1 or buffName == uabStatus_MissingBuffs.constants.strings.spirit2 then
        if not class or self.db.profile.spirit.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bspirit
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.mark1 or buffName == uabStatus_MissingBuffs.constants.strings.mark2 then
        if not class or self.db.profile.mark.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bmark
        end
	end

    if not hasbuff then
        -- dynamically added buffs
        for dynname, settings in pairs(self.db.profile.userdefined) do
            if buffName == settings.name then
                if not class or settings.classfilter[class] then
                    hasbuff = dynname
                end
                break
            end
        end
    end

    if hasbuff then
        self:UnitStatusLost(hasbuff, unit)
    end
end

function uabStatus_MissingBuffs:SpecialEvents_UnitBuffLost(unit, buffName, applications, texture, rank)
    local class = self:GetUnitClass(unit)
    local hasbuff
    
	if buffName == uabStatus_MissingBuffs.constants.strings.intellect1 or buffName == uabStatus_MissingBuffs.constants.strings.intellect2 then
        if not class or self.db.profile.intellect.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bintellect
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.fort1 or buffName == uabStatus_MissingBuffs.constants.strings.fort2 then
        if not class or self.db.profile.fort.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bfort
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.shadowprot then
        if not class or self.db.profile.shadowprot.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bshadowprot
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.spirit1 or buffName == uabStatus_MissingBuffs.constants.strings.spirit2 then
        if not class or self.db.profile.spirit.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bspirit
        end
	elseif buffName == uabStatus_MissingBuffs.constants.strings.mark1 or buffName == uabStatus_MissingBuffs.constants.strings.mark2 then
        if not class or self.db.profile.mark.classfilter[class] then
            hasbuff = uabStatus_MissingBuffs.constants.strings.bmark
        end
	end

    if not hasbuff then
        -- dynamically added buffs
        for dynname, settings in pairs(self.db.profile.userdefined) do
            if buffName == settings.name then
                if not class or settings.classfilter[class] then
                    hasbuff = dynname
                end
                break
            end
        end
    end

    if hasbuff then
        self:UnitStatusGained(hasbuff, unit)
    end
end

function uabStatus_MissingBuffs:OnEnable()
end

function uabStatus_MissingBuffs:OnDisable()
end

function uabStatus_MissingBuffs:IsStatusGained(name, unit, action)
    local class = self:GetUnitClass(unit)
    local hasbuff
    
	if name == uabStatus_MissingBuffs.constants.strings.bintellect then
        if not class or self.db.profile.intellect.classfilter[class] then
		    hasbuff = self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.intellect1) or self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.intellect2)
        end
	elseif name == uabStatus_MissingBuffs.constants.strings.bfort then
        if not class or self.db.profile.fort.classfilter[class] then
	        hasbuff = self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.fort1) or self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.fort2)
        end
	elseif name == uabStatus_MissingBuffs.constants.strings.bshadowprot then
        if not class or self.db.profile.shadowprot.classfilter[class] then
    	    hasbuff = self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.shadowprot)
        end
	elseif name == uabStatus_MissingBuffs.constants.strings.bspirit then
        if not class or self.db.profile.spirit.classfilter[class] then
	        hasbuff = self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.spirit1) or self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.spirit2)
        end
	elseif name == uabStatus_MissingBuffs.constants.strings.bmark then
        if not class or self.db.profile.mark.classfilter[class] then
    	    hasbuff = self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.mark1) or self.mixins.aura:UnitHasBuff(unit, uabStatus_MissingBuffs.constants.strings.mark2)
        end
	end

    -- dynamically added buffs
    for dynname, settings in pairs(self.db.profile.userdefined) do
        if name == dynname then
            if not class or settings.classfilter[class] then
                hasbuff = self.mixins.aura:UnitHasBuff(unit, settings.name)
            end
            break
        end
    end
    
    return not hasbuff
end

function uabStatus_MissingBuffs:OptionsForUnitStatus(name)
	if name == uabStatus_MissingBuffs.constants.strings.bintellect then
		return self.db.profile.intellect
	elseif name == uabStatus_MissingBuffs.constants.strings.bfort then
		return self.db.profile.fort
	elseif name == uabStatus_MissingBuffs.constants.strings.bshadowprot then
		return self.db.profile.shadowprot
	elseif name == uabStatus_MissingBuffs.constants.strings.bspirit then
		return self.db.profile.spirit
	elseif name == uabStatus_MissingBuffs.constants.strings.bmark then
		return self.db.profile.mark
	end

    -- dynamically added buffs
    for dynname, settings in pairs(self.db.profile.userdefined) do
        if name == dynname then
            return settings
        end
    end

end

local lastorder

function uabStatus_MissingBuffs:RemoveUserDefined(buffname)
    local args = self.core.core.options.args.unitstatus.args[self.name]

    args.args[buffname] = nil
    args.args.remove.args[buffname] = nil
    self.db.profile.userdefined[buffname] = nil
    self:UnregisterUnitStatus("Missing Buffs", buffname)

    lastorder = 50
    
    -- dynamically added buffs
    for name, settings in pairs(self.db.profile.userdefined) do
        if not settings.name then
            settings.name = name
        end

        args.args[name].order = lastorder + 10
        lastorder = args.args[name].order
    end
    
    self:SetUserDefinedBuffList()
end

function uabStatus_MissingBuffs:SetUserDefinedBuffList()
    for pos, _ in pairs(self.userdefined) do
        self.userdefined[pos] = nil
    end

    local pos = 1
    
    -- dynamically added buffs
    for name, settings in pairs(self.db.profile.userdefined) do
        self.userdefined[pos] = name
        pos = pos + 1
    end
end

function uabStatus_MissingBuffs:CreateUnitStatusOptions()
    local args = {}

    args = {
        type = 'group',
        name = "Missing Buffs",
        desc = "Configure the options for Missing Buffs unit status values",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of the Missing Buffs unit status values",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                end,
            },

            addnew = {
                name = "Add New",
                type = 'text',
                desc = "Create a new missing buff",
                usage = "<name>",
                order = 30,
                get = function() return end,
                set = function(newname)
                    self.db.profile.userdefined["Missing: "..newname] = {
                        name = newname,
                        disabled = nil,
                        priority = 50,
                        color = { r = 0, g = 0, b = 1, a = 1, },
                        classfilter = {
                            druid = true,
                            hunter = true,
                            mage = true,
                            paladin = true,
                            priest = true,
                            rogue = true,
                            shaman = true,
                            warlock = true,
                            warrior = true,
                        },
                    }

                    self:RegisterUnitStatus("Missing Buffs", "Missing: "..newname)

                    local args = self.core.core.options.args.unitstatus.args[self.name]
                                    
                    args.args["Missing: "..newname] = self:CreateUnitStatusOption(self.db.profile.userdefined["Missing: "..newname], "Missing: "..newname, "Missing: "..newname, true)
                    args.args["Missing: "..newname].order = lastorder + 10
                    lastorder = args.args["Missing: "..newname].order

                    args.args.remove.args["Missing: "..newname] = {
                        type = 'execute',
                        name = "Buff: "..newname,
                        desc = "Remove the ".."Missing: "..newname.." buff",
                        func = function()
                            self:RemoveUserDefined("Missing: "..newname)
                        end,
                    }
    
                    self:SetUserDefinedBuffList()
                end,
                validate = function(newname)
                    for name, settings in pairs(self.db.profile.userdefined) do
                        if name == "Missing: "..newname then
                            return false
                        end
                    end
                    
                    return true
                end,
                order = 2,
            },

            remove = {
                type = 'group',
                name = "Remove",
                desc = "Remove a buff",
                order = 3,
                args = {
                },
            },

            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },

        },
    }

    args.args.intellect = self:CreateUnitStatusOption(self.db.profile.intellect, uabStatus_MissingBuffs.constants.strings.bintellect, uabStatus_MissingBuffs.constants.strings.bintellect, true)
    args.args.intellect.order = 10

    args.args.fort = self:CreateUnitStatusOption(self.db.profile.fort, uabStatus_MissingBuffs.constants.strings.bfort, uabStatus_MissingBuffs.constants.strings.bfort, true)
    args.args.fort.order = 20

    args.args.spirit = self:CreateUnitStatusOption(self.db.profile.spirit, uabStatus_MissingBuffs.constants.strings.bspirit, uabStatus_MissingBuffs.constants.strings.bspirit, true)
    args.args.spirit.order = 30

    args.args.shadowprot = self:CreateUnitStatusOption(self.db.profile.shadowprot, uabStatus_MissingBuffs.constants.strings.bshadowprot, uabStatus_MissingBuffs.constants.strings.bshadowprot, true)
    args.args.shadowprot.order = 40

    args.args.mark = self:CreateUnitStatusOption(self.db.profile.mark, uabStatus_MissingBuffs.constants.strings.bmark, uabStatus_MissingBuffs.constants.strings.bmark, true)
    args.args.mark.order = 50

    lastorder = 50
    
    -- dynamically added buffs
    for name, settings in pairs(self.db.profile.userdefined) do
        if not settings.name then
            settings.name = name
        end
        
        args.args[name] = self:CreateUnitStatusOption(settings, name, name, true)
        args.args[name].order = lastorder + 10
        lastorder = args.args[name].order
    end
    
    for name, settings in pairs(self.db.profile.userdefined) do
        args.args.remove.args[name] = {
            type = 'execute',
            name = name,
            desc = "Remove the "..settings.name.." missing buff",
            order = 100,
            func = function()
                self:RemoveUserDefined(name)
            end,
        }
    end

    self:SetUserDefinedBuffList()

    return args
end

